/* *****************************
 * FSY - New version 2013-07-11
 * *****************************/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;

namespace ClarionDEMO {
	class MainClass {	
		#region properties
		private WorldServer worldServer = null;
        private ClarionAgent agent;
        String creatureId = String.Empty;
        String creatureName = String.Empty;
		Creature simulationBot = null;
		double prad = 0;
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.4");
			agent = new ClarionAgent();
            agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
            agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			
			try {          
                worldServer = new WorldServer("localhost", 4011);
                String message = worldServer.Connect();

                if (worldServer != null && worldServer.IsConnected) {                
                    Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
					// Nice behaviour ....     
					//worldServer.NewCreature(700, 500, 0, out creatureId, out creatureName);
					worldServer.NewCreature(650, 80, 0, out creatureId, out creatureName);
					// Walls, foods and jewels    
                    worldServer.NewBrick(4, 0, -5, 805, 0);
                    worldServer.NewBrick(4, 800, 0, 805, 605);
                    worldServer.NewBrick(4, -5, 600, 800, 605);
                    worldServer.NewBrick(4, -5, -5, 0, 600);
					// 
                    //Random r = new Random(DateTime.Now.Millisecond);
                    //int x_coord = 0;
                    //int y_coord = 0;

                    //for (int i = 0; i < 6; i++)
                    //{
                    //    x_coord = (int)r.Next(55, 745);
                    //    y_coord = (int)r.Next(55, 545);
                    //    worldServer.NewFood(0, x_coord, y_coord);
                    //}
					//
                    //for (int i = 0; i < 5; i++)
                    //{
                    //    for (int j = 0; j < 5; j++)
                    //    {
                    //        x_coord = (int)r.Next(55, 745);
                    //        y_coord = (int)r.Next(55, 545);
                    //        worldServer.NewJewel(j, x_coord, y_coord);
                    //    }
                    //}

                    if (!String.IsNullOrWhiteSpace(creatureId))
                    {
                        worldServer.SendStartCamera(creatureId);
                        worldServer.SendStartCreature(creatureId);
                    }

                    Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
                    agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
            }
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{
			new MainClass();
		}

        IList<Thing> agent_OnNewVisualSensorialInformation() {			
            IList<Thing> response = null;

            if (worldServer != null && worldServer.IsConnected) {            
                response = worldServer.SendGetCreatureState(creatureName);
				
				simulationBot = (Creature) response.First();
				
				prad = (Math.PI / 180) * response.First().Pitch;
			    while (prad > Math.PI) prad -= 2 * Math.PI;
			    while (prad < - Math.PI) prad += 2 * Math.PI;
            }
            return response;
        }

        void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction) {          
			Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");			
            if (worldServer != null && worldServer.IsConnected) {      
								
                switch (externalAction) {                
                    case ClarionAgentActionType.DO_NOTHING:					
                        // Do nothing as the own value says
                        break;
					
                    case ClarionAgentActionType.ROTATE:
					    //bool clockwise = false;
					    //Random rand = new Random();
					    //clockwise = Convert.ToBoolean(rand.Next(0,2));
						//Console.Out.WriteLine("Turn to .... " + clockwise +  " ...\n");
					    // If true turn clockwise, else anti-clockwise
					    //if (clockwise) {
					      double newprad = prad + Math.PI/1000;
					      while (newprad > Math.PI) newprad -= 2 * Math.PI;
			              while (newprad < - Math.PI) newprad += 2 * Math.PI;
                          worldServer.SendSetAngle(creatureId, 0, 0, newprad);
					    //}
					    //else {
					    //  double newprad = prad - Math.PI/1000;
					    //  while (newprad > Math.PI) newprad += 2 * Math.PI;
			            //  while (newprad < - Math.PI) newprad -= 2 * Math.PI;
                        //  worldServer.SendSetAngle(creatureId, 0, 0, newprad);						
					    //}
                        break;
					
				    case ClarionAgentActionType.GO_AHEAD:
						Console.Out.WriteLine("Go ahead ...... \n");
                        worldServer.SendSetAngle(creatureId, 1, 1, prad);
                        //worldServer.SendSetGoTo(creatureId, 1, 1, simulationBot.X1 + 2, simulationBot.Y1 + 2);
                        //worldServer.SendSetAngle(creatureId, 3, 3, prad);
                        break;
                    default:
                        break;
                }
            }
        }
        #endregion
	}		
}
